{ Copyright:
   Gregory Deatz
   Hoagland, Longo, Moran, Dunst & Doukas
   40 Paterson Street, Box 480
   New Brunswick, NJ 08903
   (908) 545-4717
   ----------------------------
   This product is provided as-is, with no express or implied warranties of
   any kind. Use this product at your own risk.

   It is free for you to use, distribute, modify, etc.... in any way
   you see fit; however, I would appreciate if you credit me for my
   code...

   If you have suggestions, complaints or bug reports, please contact
   Gregory Deatz - gdeatz@hlmdd.com or at the phone no. above.

   Better yet, try getting hooked up with the IB mailing list:
      interbase@mers.com
   Also try
      http://www.demon.co.uk/dtuk/dtinterbaselinks.html }
library FreeUDFLib;

(*
 * Define FULDebug if you want to output debugging information to a file.
 *  - uncomment below "commented" define
 *)

{//$define FULDebug}

uses
  SysUtils,
  Classes,
  TimeFncs in 'TimeFncs.pas',
  MathFncs in 'MathFncs.pas',
  StrFncs in 'StrFncs.pas',
  User in 'User.pas',
  Encrypt in 'Encrypt.pas',
  Misc in 'Misc.pas',
  udf_glob in 'udf_glob.pas',
  BlobFncs in 'BlobFncs.pas';

exports
  // Blob functions
  BlobAsPChar,
  BlobMaxSegmentLength,
  BlobSegmentCount,
  BlobSize,
  BlobLeft,
  BlobLine,
  BlobMid,
  BlobRight,
  StrBlob,
  BlobBinCmp,
  // Security functions
  IBPassword,
  UserMaintenance,                                      // Obsolete! (1)
  // Time/Date functions
  AddMonth,
  AddYear,
  AgeInDays,
  AgeInDaysThreshold,
  AgeInMonths,
  AgeInMonthsThreshold,
  AgeInWeeks,
  AgeInWeeksThreshold,
  CDOWLong,
  CDOWShort,
  CMonthLong,
  CMonthShort,
  DayOfMonth,
  DayOfWeek,
  DayOfYear,
  MaxDate,
  MinDate,
  Month,
  Quarter,
  StripDate,
  StripTime,
  WeekOfYear,
  WOY,
  Year,
  YearOfYear,
  // Math functions
  DollarVal,
  DoubleAbs,
  FixedPoint,
  IntegerAbs,
  IsDivisibleBy,
  Modulo,
  RoundFloat,
  Truncate,
  // String Functions
  Character,
  CRLF,
  FindNthWord, //** Not recommended.
  FindWord, //** New
  FindWordIndex, //** New
  Left,
  LineWrap,
  lrTrim,
  lTrim,
  Mid,
  PadLeft,
  PadRight,
  ProperCase,
  Right,
  rTrim,
  StringLength, //** New
  StripString, //** New
  SubStr,
  // Data formatting routines
  GenerateFormattedName,
  ValidateNameFormat,
  // Regular expressions
  ValidateRegularExpression,                            // Not implemented!
  ValidateStringInRE,                                   // Not implemented!
  // Miscellaneous functions
  CloseDebuggerOutput,
  Debug,
  IBTempPath,
  SetDebuggerOutput,
  ValidateCycleExpression,
  EvaluateCycleExpression,
  EvaluateExpression,
  // Soundex function
  GenerateSndxIndex;   // ** NEW



begin
end.

{* ************************************************************************** *}
{* * Bug fixes, acknowledgements, etc....                                   * *}
{* ************************************************************************** *}

{* Special thanks to InterBase Corp for hosting this file!                    *}

{* Encryption routines submitted by Oleg Kukarthev, a rewrite of              *}
{*   Grard Perreault's C code.                                               *}
{* Bug fix, Oct. 23, 1997 pOutStr is always 12 bytes long.                    *}
{*   Submitted by Stephen W. Boyd.                                            *}
{* MinDate submitted by Doug Chamberlin                                       *}
{* Bug fix, Dec. 30, 1997 to PadLeft                                          *}
{*   Submitted by rberryman@earthlink.net                                     *}
{* Bug fix, Jan. 1, 1997 to YearOfYear ('1/1/98' reports year=98, but week    *}
{*   is still 53 (of 97))                                                     *}
{*   Submitted by John Small (john@yagya.demon.co.uk)                         *}
{* Name formatting routines submitted by Graham Bremner                       *}
{*   (graham_bremner@hlmdd.com)                                               *}
{* Bug fix, Jan. 6, 1998 to PadRight                                          *}
{*   Submitted by rberryman@earthlink.net                                     *}
{* Bug fix, Mar. 10, 1998 to AddYear                                          *}
{*   Submitted by Serge Vannieuwenborgh (sva@netsol.be)                       *}
{* Bug fix, Sep. 9, 2000 to BlobAsPChar (use of invalid mem address crashes   *}
{*   the server, because the return value is loaded before having room.       *}
{*   Submitted by Christophe Cerbourg <decalog@wanadoo.fr>                    *}
{* Bug fix, Oct. 6, 2000 to BlobAsPchar (not null but blank blob didn't work) *}
{*   and added a test to BlobLine on nil. Moved test if (Len=0) above the     *}
{*   other test in MakeResultString so BlobMid and BlobRight can work.        *}
{*   Submitted by Claudio Valderrama.                                         *}
{* Bug fix, Feb. 11, 2000 to BlobLeft, use SetString instead of assignment of *}
{*   zero to the last char. Some detritus appeared when called from BlobMid.  *}
{*   BlobMid now uses BlobLeft instead of reading BlobAsPchar to read less.   *}
{*   Added BlobBinCmp to make a bitwise comparation of blobs, so NULL appears *}
{*   in the middle, it won't stop the comparation prematurely. The file with  *}
{*   the declarations ext_funcs.sql has been modified accordingly.            *}
{*   Added WriteDebugV to output any number of parameters in one call.        *}
{*   The initialization section in udf_glob now disposes the buffer used to   *}
{*   create the debug file immediately after being used and does not wait for *}
{*   the finalization section. Also, the file is opened with FILE_SHARE_READ. *}
{*   Remember that FULDebug can be specified in project options, too.         *}
{*   Character() in StrFncs was showing its debug information as "Truncate()" *}
{*   and it returned garbage due to conversion for @one_char to pchar, now    *}
{*   it's an array of two chars with the second being assigned NULL.          *}
{*   FindNthWord didn't have any debug information. Now it has.               *}
{*   Submitted by Claudio Valderrama.                                         *}

{* Bug fix, 5 June 2003, by KDV
   Fixed TISC_BlobGetSegment declaration (last 2 params from long to ushort).
   Fixed BlobAsPChar - bytes_left renamed to bufsize and all other corrections
   Fixed BlobLeft - same things
   Fixed BlobCMP - same things.
   Compiled with Delphi 7. Added links to VARIANTS.PAS to some implementations. 

  *}
